<?php

namespace FiloBlu\ProductUrlTools\Console\Command;

use Exception;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\CatalogUrlRewrite\Model\CategoryUrlRewriteGenerator;
use Magento\CatalogUrlRewrite\Model\CategoryUrlRewriteGeneratorFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\StoreManagerInterface;
use Magento\UrlRewrite\Model\UrlPersistInterface;
use Magento\UrlRewrite\Service\V1\Data\UrlRewrite;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class RegenerateCategoryUrlCommand
 * @package FiloBlu\ProductUrlTools\Console\Command
 */
class RegenerateCategoryUrlCommand extends Command
{

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var CollectionFactory
     */
    protected $categoryCollectionFactory;

    /**
     * @var CategoryUrlRewriteGeneratorFactory
     */
    protected $currentUrlRewritesRegeneratorFactory;
    /**
     * @var UrlPersistInterface
     */
    protected $urlPersist;

    /**
     * RegenerateCategoryUrlCommand constructor.
     * @param StoreManagerInterface $storeManager
     * @param CollectionFactory $categoryCollectionFactory
     * @param CategoryUrlRewriteGeneratorFactory $currentUrlRewritesRegeneratorFactory
     * @param UrlPersistInterface $urlPersist
     * @param null $name
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        CollectionFactory $categoryCollectionFactory,
        CategoryUrlRewriteGeneratorFactory $currentUrlRewritesRegeneratorFactory,
        UrlPersistInterface $urlPersist,

        $name = null
    )
    {
        $this->storeManager = $storeManager;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
        $this->currentUrlRewritesRegeneratorFactory = $currentUrlRewritesRegeneratorFactory;
        $this->urlPersist = $urlPersist;
        parent::__construct($name);
    }

    /**
     * Configure the command
     */
    protected function configure()
    {
        $this->setName('tools:regeneratecategory');
        $this->setDescription('Regenerate Url\'s for categories');
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @throws LocalizedException
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        /**
         *
         */
        $generator = $this->currentUrlRewritesRegeneratorFactory->create();
        foreach ($this->storeManager->getStores() as $store) {

            $categories = $this->categoryCollectionFactory->create()
                ->setStore($store->getId())
                ->addAttributeToSelect(['name', 'url_path', 'url_key']);

            foreach ($categories as $category) {
                if (in_array((int)$category->getLevel(), [0, 1], true)) {
                    continue;
                }
                $category->setStoreId($store->getId());
                $output->writeln(sprintf('Store (%d) %s - Category %s', $store->getId(),  $store->getName() ,$category->getName()));

                $urls = $generator->generate($category);
                try {
                    $this->urlPersist->deleteByData([
                        UrlRewrite::ENTITY_ID => $category->getId(),
                        UrlRewrite::ENTITY_TYPE => CategoryUrlRewriteGenerator::ENTITY_TYPE,
                        UrlRewrite::REDIRECT_TYPE => 0,
                        UrlRewrite::STORE_ID => $store->getId()
                    ]);
                    $this->urlPersist->replace($this->filterEmptyRequestPaths($urls));
                } catch (Exception $e) {
                    $output->writeln(sprintf('<error>Duplicated url for store ID %d, category %d (%s) - %s Generated URLs:' . PHP_EOL . '%s</error>' . PHP_EOL, $store->getId(), $category->getId(), $category->getName(), $e->getMessage(), implode(PHP_EOL, array_keys($urls))));
                }

            }
        }

    }

    /**
     * Remove entries with request_path='' to prevent error 404 for "http://site.com/" address.
     *
     * @param UrlRewrite[] $newUrls
     * @return UrlRewrite[]
     */
    private function filterEmptyRequestPaths($newUrls)
    {
        $result = [];
        foreach ($newUrls as $key => $url) {
            $requestPath = $url->getRequestPath();
            if (!empty($requestPath)) {
                $result[$key] = $url;
            }
        }
        return $result;
    }
}